@@ -315,12 +315,18 @@ class PythonParser:
         to_add = set()
         for arc in arcs:
             if arc in self._with_jump_fixers:
+                start = arc[0]
+                to_remove.add(arc)
                 start_next, prev_next = self._with_jump_fixers[arc]
-                if start_next in arcs:
-                    to_add.add(prev_next)
-                    to_remove.add(arc)
+                while start_next in self._with_jump_fixers:
                     to_remove.add(start_next)
-        return (set(arcs) | to_add) - to_remove
+                    start_next, prev_next = self._with_jump_fixers[start_next]
+                    to_remove.add(prev_next)
+                to_add.add((start, prev_next[1]))
+                to_remove.add(arc)
+                to_remove.add(start_next)
+        arcs = (set(arcs) | to_add) - to_remove
+        return arcs
 
     @functools.lru_cache
     def exit_counts(self) -> dict[TLineNo, int]:
@@ -719,7 +725,7 @@ class AstArcAnalyzer:
         if node.body:
             exits = self.process_body(node.body)
             for xit in exits:
-                self.add_arc(xit.lineno, -start, xit.cause, "didn't exit the module")
+                self.add_arc(xit.lineno, -start, xit.cause, "exit the module")
         else:
             # Empty module.
             self.add_arc(start, -start)
